'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _validate = require('./validate');

var _validate2 = _interopRequireDefault(_validate);

var _context = require('./context');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

//Checks if another field has a rule depending on this field
var hasDependentRule = function hasDependentRule(name, rules) {
  return Object.values(rules).flatMap(function (x) {
    return x;
  }).find(function (rule) {
    return rule.needs && rule.needs.includes(name);
  });
};

var useField = function useField(name) {
  var _React$useContext = _react2.default.useContext(_context.FormContext),
      errors = _React$useContext.errors,
      rules = _React$useContext.rules,
      values = _React$useContext.values,
      setErrors = _React$useContext.setErrors,
      hasBlurred = _React$useContext.hasBlurred,
      updateValue = _React$useContext.updateValue,
      valuesBlurred = _React$useContext.valuesBlurred;

  var value = values[name];
  // Pulling out just this field's errors
  var fieldErrors = errors.filter(function (error) {
    return error.name === name;
  }).map(function (error) {
    return error.message;
  });

  //Args needed for validation
  var validationProps = {
    values: values,
    rules: rules,
    valuesBlurred: valuesBlurred,
    setErrors: setErrors,
    errors: errors
  };

  var handleChange = function handleChange(value) {
    /*
      If this field 
      - has errors
      - blurred before
      - has dependant rules
      check validation on change
    */
    if (fieldErrors.length || valuesBlurred[name] || hasDependentRule(name, rules)) (0, _validate2.default)(_extends({}, validationProps, {
      values: _extends({}, values, _defineProperty({}, name, value)),
      valuesBlurred: _extends({}, valuesBlurred, _defineProperty({}, name, true))
    }));

    updateValue(name, value);
  };

  /*
    When blurring for the first time, 
    lets send hasBlurred, and validate the field
  */
  var handleBlur = function handleBlur() {
    hasBlurred(name);
    (0, _validate2.default)(_extends({}, validationProps, {
      valuesBlurred: _extends({}, valuesBlurred, _defineProperty({}, name, true))
    }));
  };

  return {
    handleBlur: handleBlur,
    handleChange: handleChange,
    value: value,
    errors: fieldErrors.length ? fieldErrors : null
  };
};

exports.default = useField;